/* 
 * k-nearest neighbors with squared Mahalanobis distance metric
 * MEMOCODE 2014 Design Contest
 * Peter Milder, peter.milder@stonybrook.edu
 *
 * For more information, please see README.TXT and
 * http://www.ece.stonybrook.edu/~pmilder/memocode/
 *
 * This program provides code to compare output data with the provided 
 * reference output data
 *
 * Usage: ./eval_results refDists refInd outDists outInd
 * See README.TXT for input/output specification, or run: 
 *     make runsmall
 * to compile and run the small example. 
 */

/*
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

int countBits(long int x) {
	int b=0;
	while (x) {
		b++;
		x = x >> 1;
	}
	return b;
}

// usage: ./eval_results refDists refInd outDists outInd
int main(int argc, char *argv[]) {

	FILE *fRefDists;
	FILE *fRefInds;
	FILE *fOutDists;
	FILE *fOutInd;

	if (argc != 5) {
		printf("Usage: ./eval_results refDists refInd outDists outInd\n");
		return 1;
	}
	
	if ((fRefDists = fopen(argv[1], "r")) == NULL) {
		printf("Cannot open file %s\n", argv[1]);
		assert(0);
	}
	if ((fRefInds = fopen(argv[2], "r")) == NULL) {
		printf("Cannot open file %s\n", argv[2]);
		assert(0);
	}
	if ((fOutDists = fopen(argv[3], "r")) == NULL) {
		printf("Cannot open file %s\n", argv[3]);
		assert(0);
	}
	if ((fOutInd = fopen(argv[4], "r")) == NULL) {
		printf("Cannot open file %s\n", argv[4]);
		assert(0);
	}
	
	int indexMismatch = 0;
	int refInd, outInd;

	// Look for differences in index files and check
	// files are same length.
	while (fscanf(fRefInds, "%d", &refInd) != EOF) {
		if (fscanf(fOutInd, "%d", &outInd) == EOF) {
			printf("Error, index output file length does not match length of reference\n");
			assert(0);
		}
		if (refInd != outInd) {
			//printf("Error: %d vs %d\n", refInd, outInd);
			indexMismatch++;
		}	
	}
	if (fscanf(fOutInd, "%d", &outInd) != EOF) {
		printf("Error, index output file length does not match length of reference\n");
		assert(0);
	}	
	
	fclose(fOutInd);
	fclose(fRefInds);
	
	long int worstDistMismatch = 0;
	long int refDist, outDist;
	int distMismatch = 0;

	// Look for differences in distance files and check
	// files are same length.
	while (fscanf(fRefDists, "%ld", &refDist) != EOF) {
		if (fscanf(fOutDists, "%ld", &outDist) == EOF) {
			printf("Error, distance output file length does not match length of reference\n");
			assert(0);
		}
		if (refDist != outDist) {
			//printf("Error: %ld vs %ld\n", refDist, outDist);
			distMismatch++;
			if (labs(refDist - outDist) > worstDistMismatch) {
				//printf("New worst mismatch = %ld\n", labs(refDist - outDist));
				worstDistMismatch = labs(refDist - outDist);
			}
		}	
	}
	if (fscanf(fOutDists, "%ld", &outDist) != EOF) {
		printf("Error, distance output file length does not match length of reference\n");
		assert(0);
	}	
	
	if (indexMismatch || distMismatch) {
		printf("Found %d index mismatches\n", indexMismatch);
		printf("Found %d distance mismatches\n", distMismatch);
		printf("Worst distance mismatch was %ld (%d bits)\n", worstDistMismatch, countBits(worstDistMismatch));
	}
	else {
		printf("All tests passed.\n");
	}
	fclose(fOutDists);
	fclose(fRefDists);
	
    return 0;
}

